# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import numpy as np

from ...common.result import BaseResult
from .visualizer_3d import Visualizer3D


class BEV3DDetResult(BaseResult):
    """Base class for computer vision results."""

    def __init__(self, data: dict) -> None:
        """
        Initialize the BaseCVResult.

        Args:
            data (dict): The initial data.

        Raises:
            AssertionError: If the required key (`BaseCVResult.INPUT_IMG_KEY`) are not found in the data.
        """

        super().__init__(data)

    def visualize(self, save_path: str, show: bool) -> None:
        # input point cloud
        assert "input_path" in self.keys(), "input_path is not found in the data"
        points = np.fromfile(self["input_path"], dtype=np.float32)
        points = points.reshape(-1, 5)
        points = points[:, :4]

        # detection result
        result = dict()
        assert "boxes_3d" in self.keys(), "boxes_3d is not found in the data"
        result["bbox3d"] = self["boxes_3d"]
        assert "scores_3d" in self.keys(), "scores_3d is not found in the data"
        result["scores"] = self["scores_3d"]
        assert "labels_3d" in self.keys(), "labels_3d is not found in the data"
        result["labels"] = self["labels_3d"]

        if save_path is not None:
            # save result for local visualization
            if not os.path.exists(save_path):
                os.makedirs(save_path)
            np.save(os.path.join(save_path, "results.npy"), result)
            np.save(os.path.join(save_path, "points.npy"), points)

        if show:
            # visualize
            score_threshold = 0.25
            vis = Visualizer3D()
            vis.draw_results(points, result, score_threshold)

        return



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys
import tempfile

from packaging.requirements import Requirement

from . import logging


def install_packages_from_requirements_file(
    requirements_file_path, pip_install_opts=None
):
    from .deps import DEP_SPECS

    # TODO: Precompute or cache the constraints
    with tempfile.NamedTemporaryFile("w", suffix=".txt", delete=False) as f:
        for reqs in DEP_SPECS.values():
            for req in reqs:
                req = Requirement(req)
                if req.marker and not req.marker.evaluate():
                    continue
                if req.url:
                    req = f"{req.name}@{req.url}"
                else:
                    req = f"{req.name}{req.specifier}"
                f.write(req + "\n")
        constraints_file_path = f.name

    args = [
        sys.executable,
        "-m",
        "pip",
        "install",
        "-c",
        constraints_file_path,
        *(pip_install_opts or []),
        "-r",
        requirements_file_path,
    ]
    logging.debug("Command: %s", args)

    try:
        return subprocess.check_call(args)
    finally:
        os.unlink(constraints_file_path)


def install_packages(requirements, pip_install_opts=None):
    with tempfile.NamedTemporaryFile("w", suffix=".txt", delete=False) as f:
        for req in requirements:
            f.write(req + "\n")
        reqs_file_path = f.name
    try:
        return install_packages_from_requirements_file(
            reqs_file_path, pip_install_opts=pip_install_opts
        )
    finally:
        os.unlink(reqs_file_path)


def uninstall_packages(packages, pip_uninstall_opts=None):
    args = [
        sys.executable,
        "-m",
        "pip",
        "uninstall",
        "-y",
        *(pip_uninstall_opts or []),
        *packages,
    ]
    logging.debug("Command: %s", args)
    return subprocess.check_call(args)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .configuration_utils import GenerationConfig
from .logits_process import (
    ForcedBOSTokenLogitsProcessor,
    ForcedEOSTokenLogitsProcessor,
    HammingDiversityLogitsProcessor,
    LogitsProcessor,
    LogitsProcessorList,
    MinLengthLogitsProcessor,
    RepetitionPenaltyLogitsProcessor,
    TopKProcess,
    TopPProcess,
)
from .stopping_criteria import (
    MaxLengthCriteria,
    MaxTimeCriteria,
    StoppingCriteria,
    StoppingCriteriaList,
    validate_stopping_criteria,
)
from .utils import GenerationMixin



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from os import PathLike
from pathlib import Path
from typing import Any, Dict, Optional, Protocol, Union, runtime_checkable

from pydantic import BaseModel, Discriminator, SecretStr, TypeAdapter
from typing_extensions import Annotated, Literal, assert_never

from ....utils.deps import class_requires_deps

__all__ = [
    "InMemoryStorageConfig",
    "FileSystemStorageConfig",
    "BOSConfig",
    "FileStorageConfig",
    "SupportsGetURL",
    "Storage",
    "InMemoryStorage",
    "FileSystemStorage",
    "BOS",
    "create_storage",
]


class InMemoryStorageConfig(BaseModel):
    type: Literal["memory"] = "memory"


class FileSystemStorageConfig(BaseModel):
    directory: Union[str, PathLike]

    type: Literal["file_system"] = "file_system"


class BOSConfig(BaseModel):
    endpoint: str
    ak: SecretStr
    sk: SecretStr
    bucket_name: str
    key_prefix: Optional[str] = None
    connection_timeout_in_mills: Optional[int] = None

    type: Literal["bos"] = "bos"


FileStorageConfig = Annotated[
    Union[InMemoryStorageConfig, FileSystemStorageConfig, BOSConfig],
    Discriminator("type"),
]


@runtime_checkable
class SupportsGetURL(Protocol):
    def get_url(self, key: str) -> str: ...


class Storage(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def get(self, key: str) -> bytes:
        raise NotImplementedError

    @abc.abstractmethod
    def set(self, key: str, value: bytes) -> None:
        raise NotImplementedError

    @abc.abstractmethod
    def delete(self, key: str) -> None:
        raise NotImplementedError


class InMemoryStorage(Storage):
    def __init__(self, config: InMemoryStorageConfig) -> None:
        super().__init__()
        self._data: Dict[str, bytes] = {}

    def get(self, key: str) -> bytes:
        return self._data[key]

    def set(self, key: str, value: bytes) -> None:
        self._data[key] = value

    def delete(self, key: str) -> None:
        del self._data[key]


class FileSystemStorage(Storage):
    def __init__(self, config: FileSystemStorageConfig) -> None:
        super().__init__()
        self._directory = Path(config.directory)
        self._directory.mkdir(exist_ok=True)

    def get(self, key: str) -> bytes:
        with open(self._get_file_path(key), "rb") as f:
            contents = f.read()
        return contents

    def set(self, key: str, value: bytes) -> None:
        path = self._get_file_path(key)
        path.parent.mkdir(exist_ok=True)
        with open(path, "wb") as f:
            f.write(value)

    def delete(self, key: str) -> None:
        file_path = self._get_file_path(key)
        file_path.unlink(missing_ok=True)

    def _get_file_path(self, key: str) -> Path:
        return self._directory / key


@class_requires_deps("bce-python-sdk")
class BOS(Storage):
    def __init__(self, config: BOSConfig) -> None:
        from baidubce.auth.bce_credentials import BceCredentials
        from baidubce.bce_client_configuration import BceClientConfiguration
        from baidubce.services.bos.bos_client import BosClient

        super().__init__()

        bos_cfg = BceClientConfiguration(
            credentials=BceCredentials(
                config.ak.get_secret_value(), config.sk.get_secret_value()
            ),
            endpoint=config.endpoint,
            connection_timeout_in_mills=config.connection_timeout_in_mills,
        )
        self._client = BosClient(bos_cfg)
        self._bucket_name = config.bucket_name
        self._key_prefix = config.key_prefix

    def get(self, key: str) -> bytes:
        key = self._get_full_key(key)
        return self._client.get_object_as_string(bucket_name=self._bucket_name, key=key)

    def set(self, key: str, value: bytes) -> None:
        key = self._get_full_key(key)
        self._client.put_object_from_string(
            bucket=self._bucket_name, key=key, data=value
        )

    def delete(self, key: str) -> None:
        key = self._get_full_key(key)
        self._client.delete_object(bucket_name=self._bucket_name, key=key)

    def get_url(self, key: str) -> str:
        key = self._get_full_key(key)
        return self._client.generate_pre_signed_url(
            self._bucket_name, key, expiration_in_seconds=-1
        ).decode("ascii")

    def _get_full_key(self, key: str) -> str:
        if self._key_prefix:
            return f"{self._key_prefix}/{key}"
        return key


def create_storage(dic: Dict[str, Any], /) -> Storage:
    config = TypeAdapter(FileStorageConfig).validate_python(dic)
    if config.type == "memory":
        return InMemoryStorage(config)
    elif config.type == "file_system":
        return FileSystemStorage(config)
    elif config.type == "bos":
        return BOS(config)
    else:
        assert_never(config)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from typing import Any, Dict, List, Tuple, Union

import pandas as pd

from ....modules.ts_anomaly_detection.model_list import MODELS
from ...common.batch_sampler import TSBatchSampler
from ...common.reader import ReadTS
from ..base import BasePredictor
from ..common import (
    BuildTSDataset,
    TimeFeature,
    TSCutOff,
    TSNormalize,
    TStoArray,
    TStoBatch,
)
from .processors import GetAnomaly
from .result import TSAdResult


class TSAdPredictor(BasePredictor):
    """TSAdPredictor that inherits from BasePredictor."""

    entities = MODELS

    def __init__(self, *args: List, **kwargs: Dict) -> None:
        """Initializes TSAdPredictor.

        Args:
            *args: Arbitrary positional arguments passed to the superclass.
            **kwargs: Arbitrary keyword arguments passed to the superclass.
        """
        super().__init__(*args, **kwargs)
        self.preprocessors, self.infer, self.postprocessors = self._build()

    def _build_batch_sampler(self) -> TSBatchSampler:
        """Builds and returns an ImageBatchSampler instance.

        Returns:
            ImageBatchSampler: An instance of ImageBatchSampler.
        """
        return TSBatchSampler()

    def _get_result_class(self) -> type:
        """Returns the result class, TopkResult.

        Returns:
            type: The TopkResult class.
        """
        return TSAdResult

    def _build(self) -> Tuple:
        """Build the preprocessors, inference engine, and postprocessors based on the configuration.

        Returns:
            tuple: A tuple containing the preprocessors, inference engine, and postprocessors.
        """
        preprocessors = {
            "ReadTS": ReadTS(),
            "TSCutOff": TSCutOff(self.config["size"]),
        }

        if self.config.get("scale", None):
            scaler_file_path = os.path.join(self.model_dir, "scaler.pkl")
            if not os.path.exists(scaler_file_path):
                raise Exception(f"Cannot find scaler file: {scaler_file_path}")
            preprocessors["TSNormalize"] = TSNormalize(
                scaler_file_path, self.config["info_params"]
            )

        preprocessors["BuildTSDataset"] = BuildTSDataset(self.config["info_params"])

        if self.config.get("time_feat", None):
            preprocessors["TimeFeature"] = TimeFeature(
                self.config["info_params"],
                self.config["size"],
                self.config["holiday"],
            )
        preprocessors["TStoArray"] = TStoArray(self.config["input_data"])
        preprocessors["TStoBatch"] = TStoBatch()
        infer = self.create_static_infer()
        postprocessors = {}
        postprocessors["GetAnomaly"] = GetAnomaly(
            self.config["model_threshold"], self.config["info_params"]
        )
        return preprocessors, infer, postprocessors

    def process(self, batch_data: List[Union[str, pd.DataFrame]]) -> Dict[str, Any]:
        """
        Process a batch of data through the preprocessing, inference, and postprocessing.

        Args:
            batch_data (List[Union[str, pd.DataFrame], ...]): A batch of input data (e.g., image file paths).

        Returns:
            dict: A dictionary containing the input path, raw image, class IDs, scores, and label names for every instance of the batch. Keys include 'input_path', 'input_img', 'class_ids', 'scores', and 'label_names'.
        """

        batch_raw_ts = self.preprocessors["ReadTS"](ts_list=batch_data.instances)
        batch_cutoff_ts = self.preprocessors["TSCutOff"](ts_list=batch_raw_ts)

        if "TSNormalize" in self.preprocessors:
            batch_ts = self.preprocessors["TSNormalize"](ts_list=batch_cutoff_ts)
            batch_input_ts = self.preprocessors["BuildTSDataset"](ts_list=batch_ts)
        else:
            batch_input_ts = self.preprocessors["BuildTSDataset"](
                ts_list=batch_cutoff_ts
            )

        if "TimeFeature" in self.preprocessors:
            batch_ts = self.preprocessors["TimeFeature"](ts_list=batch_input_ts)
            batch_ts = self.preprocessors["TStoArray"](ts_list=batch_ts)
        else:
            batch_ts = self.preprocessors["TStoArray"](ts_list=batch_input_ts)

        x = self.preprocessors["TStoBatch"](ts_list=batch_ts)
        batch_preds = self.infer(x=x)

        batch_ts_preds = self.postprocessors["GetAnomaly"](
            ori_ts_list=batch_input_ts, pred_list=batch_preds
        )
        return {
            "input_path": batch_data.input_paths,
            "input_ts": batch_raw_ts,
            "anomaly": batch_ts_preds,
        }



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ..base import BaseTrainer
from .model_list import MODELS


class TableRecTrainer(BaseTrainer):
    """Table Recognition Model Trainer"""

    entities = MODELS

    def update_config(self):
        """update training config"""
        if self.train_config.log_interval:
            self.pdx_config.update_log_interval(self.train_config.log_interval)
        if self.train_config.eval_interval:
            self.pdx_config._update_eval_interval_by_epoch(
                self.train_config.eval_interval
            )
        if self.train_config.save_interval:
            self.pdx_config.update_save_interval(self.train_config.save_interval)

        self.pdx_config.update_dataset(
            self.global_config.dataset_dir, "PubTabTableRecDataset"
        )
        if self.train_config.pretrain_weight_path:
            self.pdx_config.update_pretrained_weights(
                self.train_config.pretrain_weight_path
            )
        if self.train_config.batch_size is not None:
            self.pdx_config.update_batch_size(self.train_config.batch_size)
        if self.train_config.learning_rate is not None:
            self.pdx_config.update_learning_rate(self.train_config.learning_rate)
        if self.train_config.epochs_iters is not None:
            self.pdx_config._update_epochs(self.train_config.epochs_iters)
        if (
            self.train_config.resume_path is not None
            and self.train_config.resume_path != ""
        ):
            self.pdx_config._update_checkpoints(self.train_config.resume_path)
        if self.global_config.output is not None:
            self.pdx_config._update_output_dir(self.global_config.output)

    def get_train_kwargs(self) -> dict:
        """get key-value arguments of model training function

        Returns:
            dict: the arguments of training function.
        """
        return {
            "device": self.get_device(),
            "dy2st": self.train_config.get("dy2st", False),
            "amp": self.train_config.get("amp", "OFF"),  # amp support 'O1', 'O2', 'OFF'
        }



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
from pathlib import Path
from typing import Dict

import numpy as np
from PIL import Image, ImageDraw

from ...common.result import BaseCVResult, HtmlMixin, JsonMixin, XlsxMixin


class SingleTableRecognitionResult(BaseCVResult, HtmlMixin, XlsxMixin):
    """single table recognition result"""

    def __init__(self, data: Dict) -> None:
        super().__init__(data)
        HtmlMixin.__init__(self)
        XlsxMixin.__init__(self)

    def _get_input_fn(self):
        fn = super()._get_input_fn()
        if (page_idx := self["page_index"]) is not None:
            fp = Path(fn)
            stem, suffix = fp.stem, fp.suffix
            return f"{stem}_{page_idx}{suffix}"
        else:
            return fn

    def _to_html(self) -> Dict[str, str]:
        """Converts the prediction to its corresponding HTML representation.

        Returns:
            Dict[str, str]: The str type HTML representation result.
        """
        return {"pred": self["pred_html"]}

    def _to_xlsx(self) -> Dict[str, str]:
        """Converts the prediction HTML to an XLSX file path.

        Returns:
            str: The path to the XLSX file containing the prediction data.
        """
        return {"pred": self["pred_html"]}

    def _to_str(self, *args, **kwargs) -> Dict[str, str]:
        """Converts the instance's attributes to a dictionary and then to a string.

        Args:
            *args: Additional positional arguments passed to the base class method.
            **kwargs: Additional keyword arguments passed to the base class method.

        Returns:
            Dict[str, str]: A dictionary with the instance's attributes converted to strings.
        """
        data = {}
        data["cell_box_list"] = self["cell_box_list"]
        data["pred_html"] = self["pred_html"]
        data["table_ocr_pred"] = self["table_ocr_pred"]
        return JsonMixin._to_str(data, *args, **kwargs)

    def _to_json(self, *args, **kwargs) -> Dict[str, str]:
        """
        Converts the object's data to a JSON dictionary.

        Args:
            *args: Positional arguments passed to the JsonMixin._to_json method.
            **kwargs: Keyword arguments passed to the JsonMixin._to_json method.

        Returns:
            Dict[str, str]: A dictionary containing the object's data in JSON format.
        """
        data = {}
        data["cell_box_list"] = self["cell_box_list"]
        data["pred_html"] = self["pred_html"]
        data["table_ocr_pred"] = self["table_ocr_pred"]
        return JsonMixin._to_json(data, *args, **kwargs)


class TableRecognitionResult(BaseCVResult, HtmlMixin, XlsxMixin):
    """Table Recognition Result"""

    def __init__(self, data: Dict) -> None:
        super().__init__(data)
        HtmlMixin.__init__(self)
        XlsxMixin.__init__(self)
        if self["layout_det_res"] is None:
            self["layout_det_res"] = {}

    def _get_input_fn(self):
        fn = super()._get_input_fn()
        if (page_idx := self["page_index"]) is not None:
            fp = Path(fn)
            stem, suffix = fp.stem, fp.suffix
            return f"{stem}_{page_idx}{suffix}"
        else:
            return fn

    def _to_img(self) -> Dict[str, np.ndarray]:
        res_img_dict = {}
        layout_det_res = self["layout_det_res"]
        if len(layout_det_res) > 0:
            res_img_dict["layout_det_res"] = layout_det_res.img["res"]

        model_settings = self["model_settings"]
        if model_settings["use_doc_preprocessor"]:
            res_img_dict.update(**self["doc_preprocessor_res"].img)

        res_img_dict.update(**self["overall_ocr_res"].img)

        if len(self["table_res_list"]) > 0:
            table_cell_img = Image.fromarray(
                copy.deepcopy(self["doc_preprocessor_res"]["output_img"][:, :, ::-1])
            )
            table_draw = ImageDraw.Draw(table_cell_img)
            rectangle_color = (255, 0, 0)
            for sno in range(len(self["table_res_list"])):
                table_res = self["table_res_list"][sno]
                cell_box_list = table_res["cell_box_list"]
                for box in cell_box_list:
                    x1, y1, x2, y2 = [int(pos) for pos in box]
                    table_draw.rectangle(
                        [x1, y1, x2, y2], outline=rectangle_color, width=2
                    )
            res_img_dict["table_cell_img"] = table_cell_img
        return res_img_dict

    def _to_str(self, *args, **kwargs) -> Dict[str, str]:
        """Converts the instance's attributes to a dictionary and then to a string.

        Args:
            *args: Additional positional arguments passed to the base class method.
            **kwargs: Additional keyword arguments passed to the base class method.

        Returns:
            Dict[str, str]: A dictionary with the instance's attributes converted to strings.
        """
        data = {}
        data["input_path"] = self["input_path"]
        data["page_index"] = self["page_index"]
        data["model_settings"] = self["model_settings"]
        if self["model_settings"]["use_doc_preprocessor"]:
            data["doc_preprocessor_res"] = self["doc_preprocessor_res"].str["res"]
        if len(self["layout_det_res"]) > 0:
            data["layout_det_res"] = self["layout_det_res"].str["res"]
        data["overall_ocr_res"] = self["overall_ocr_res"].str["res"]
        data["table_res_list"] = []
        for sno in range(len(self["table_res_list"])):
            table_res = self["table_res_list"][sno]
            data["table_res_list"].append(table_res.str["res"])
        return JsonMixin._to_str(data, *args, **kwargs)

    def _to_json(self, *args, **kwargs) -> Dict[str, str]:
        """
        Converts the object's data to a JSON dictionary.

        Args:
            *args: Positional arguments passed to the JsonMixin._to_json method.
            **kwargs: Keyword arguments passed to the JsonMixin._to_json method.

        Returns:
            Dict[str, str]: A dictionary containing the object's data in JSON format.
        """
        data = {}
        data["input_path"] = self["input_path"]
        data["page_index"] = self["page_index"]
        data["model_settings"] = self["model_settings"]
        if self["model_settings"]["use_doc_preprocessor"]:
            data["doc_preprocessor_res"] = self["doc_preprocessor_res"].json["res"]
        if len(self["layout_det_res"]) > 0:
            data["layout_det_res"] = self["layout_det_res"].json["res"]
        data["overall_ocr_res"] = self["overall_ocr_res"].json["res"]
        data["table_res_list"] = []
        for sno in range(len(self["table_res_list"])):
            table_res = self["table_res_list"][sno]
            data["table_res_list"].append(table_res.json["res"])
        return JsonMixin._to_json(data, *args, **kwargs)

    def _to_html(self) -> Dict[str, str]:
        """Converts the prediction to its corresponding HTML representation.

        Returns:
            Dict[str, str]: The str type HTML representation result.
        """
        res_html_dict = {}
        for sno in range(len(self["table_res_list"])):
            table_res = self["table_res_list"][sno]
            table_region_id = table_res["table_region_id"]
            key = f"table_{table_region_id}"
            res_html_dict[key] = table_res.html["pred"]
            res_html_dict[key] = res_html_dict[key].replace(
                "<table>", '<table border="1">'
            )
        return res_html_dict

    def _to_xlsx(self) -> Dict[str, str]:
        """Converts the prediction HTML to an XLSX file path.

        Returns:
            Dict[str, str]: The str type XLSX representation result.
        """
        res_xlsx_dict = {}
        for sno in range(len(self["table_res_list"])):
            table_res = self["table_res_list"][sno]
            table_region_id = table_res["table_region_id"]
            key = f"table_{table_region_id}"
            res_xlsx_dict[key] = table_res.xlsx["pred"]
        return res_xlsx_dict



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ...utils import logging
from ..base import BaseTrainer
from .model_list import MODELS


class DetTrainer(BaseTrainer):
    """Object Detection Model Trainer"""

    entities = MODELS

    def _update_dataset(self):
        """update dataset settings"""
        metric = self.pdx_config.metric if "metric" in self.pdx_config else "COCO"
        data_fields = (
            self.pdx_config.TrainDataset["data_fields"]
            if "data_fields" in self.pdx_config.TrainDataset
            else None
        )

        self.pdx_config.update_dataset(
            self.global_config.dataset_dir,
            "COCODetDataset",
            data_fields=data_fields,
            metric=metric,
        )

    def update_config(self):
        """update training config"""
        if self.train_config.log_interval:
            self.pdx_config.update_log_interval(self.train_config.log_interval)
        if self.train_config.eval_interval:
            self.pdx_config.update_eval_interval(self.train_config.eval_interval)

        self._update_dataset()

        if self.train_config.num_classes is not None:
            self.pdx_config.update_num_class(self.train_config.num_classes)
        if (
            self.train_config.pretrain_weight_path
            and self.train_config.pretrain_weight_path != ""
        ):
            self.pdx_config.update_pretrained_weights(
                self.train_config.pretrain_weight_path
            )
        if self.train_config.batch_size is not None:
            self.pdx_config.update_batch_size(self.train_config.batch_size)
        if self.train_config.learning_rate is not None:
            self.pdx_config.update_learning_rate(self.train_config.learning_rate)
        if self.train_config.epochs_iters is not None:
            self.pdx_config.update_epochs(self.train_config.epochs_iters)
            epochs_iters = self.train_config.epochs_iters
        else:
            epochs_iters = self.pdx_config.get_epochs_iters()
        if self.train_config.warmup_steps is not None:
            self.pdx_config.update_warmup_steps(self.train_config.warmup_steps)
        if self.global_config.output is not None:
            self.pdx_config.update_save_dir(self.global_config.output)

        if "PicoDet" in self.global_config.model:
            assigner_epochs = max(int(epochs_iters / 10), 1)
            try:
                self.pdx_config.update_static_assigner_epochs(assigner_epochs)
            except Exception:
                logging.info(
                    f"The model({self.global_config.model}) don't support to update_static_assigner_epochs!"
                )

    def get_train_kwargs(self) -> dict:
        """get key-value arguments of model training function

        Returns:
            dict: the arguments of training function.
        """
        train_args = {"device": self.get_device()}
        if (
            self.train_config.resume_path is not None
            and self.train_config.resume_path != ""
        ):
            train_args["resume_path"] = self.train_config.resume_path
        train_args["dy2st"] = self.train_config.get("dy2st", False)
        # amp support 'O1', 'O2', 'OFF'
        train_args["amp"] = self.train_config.get("amp", "OFF")
        return train_args



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os

from .....utils.file_interface import custom_open


def convert(input_dir):
    """
    Convert json in file into imagenet format.
    """
    label_path = os.path.join(input_dir, "flags.txt")
    label_dict = {}
    label_content = []
    with custom_open(label_path, "r") as f:
        lines = f.readlines()
        for idx, line in enumerate(lines):
            line = line.strip()
            label_dict[line] = str(idx)
            label_content.append(f"{str(idx)} {line}\n")
    with custom_open(os.path.join(input_dir, "label.txt"), "w") as f:
        f.write("".join(label_content))
    anno_path = os.path.join(input_dir, "annotations")
    os.listdir()
    train_list = os.path.join(input_dir, "train.txt")
    val_list = os.path.join(input_dir, "val.txt")
    label_info = []
    for json_file in os.listdir(anno_path):
        with custom_open(os.path.join(anno_path, json_file), "r") as f:
            data = json.load(f)
            file_name = os.path.join("images", data["imagePath"].strip().split("/")[2])
            for label, value in data["flags"].items():
                if value:
                    label_info.append(f"{file_name} {label_dict[label]}\n")
    with custom_open(train_list, "w") as file:
        file.write("".join(label_info))
    with custom_open(val_list, "w") as file:
        pass



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ....utils.errors import raise_unsupported_api_error
from ...base import BaseRunner
from ...base.utils.arg import gather_opts_args
from ...base.utils.subprocess import CompletedProcess


class TSRunner(BaseRunner):
    """TS Runner"""

    def train(
        self,
        config_path: str,
        cli_args: list,
        device: str,
        ips: str,
        save_dir: str,
        do_eval=True,
    ) -> CompletedProcess:
        """train model

        Args:
            config_path (str): the config file path used to train.
            cli_args (list): the additional parameters.
            device (str): the training device.
            ips (str): the ip addresses of nodes when using distribution.
            save_dir (str): the directory path to save training output.
            do_eval (bool, optional): whether or not to evaluate model during training. Defaults to True.

        Returns:
            CompletedProcess: the result of training subprocess execution.
        """
        args, env = self.distributed(device, ips, log_dir=save_dir)
        cli_args = self._gather_opts_args(cli_args)
        cmd = [*args, "tools/train.py", "--config", config_path, *cli_args]
        return self.run_cmd(
            cmd,
            env=env,
            switch_wdir=True,
            echo=True,
            silent=False,
            capture_output=True,
            log_path=self._get_train_log_path(save_dir),
        )

    def evaluate(
        self, config_path: str, cli_args: list, device: str, ips: str
    ) -> CompletedProcess:
        """run model evaluating

        Args:
            config_path (str): the config file path used to evaluate.
            cli_args (list): the additional parameters.
            device (str): the evaluating device.
            ips (str): the ip addresses of nodes when using distribution.

        Returns:
            CompletedProcess: the result of evaluating subprocess execution.
        """
        args, env = self.distributed(device, ips)
        cli_args = self._gather_opts_args(cli_args)
        cmd = [*args, "tools/val.py", "--config", config_path, *cli_args]

        cp = self.run_cmd(
            cmd, env=env, switch_wdir=True, echo=True, silent=False, capture_output=True
        )
        if cp.returncode == 0:
            metric_dict = _extract_eval_metrics(cp.stderr)
            cp.metrics = metric_dict
        return cp

    def predict(
        self, config_path: str, cli_args: list, device: str
    ) -> CompletedProcess:
        """run predicting using dynamic mode

        Args:
            config_path (str): the config file path used to predict.
            cli_args (list): the additional parameters.
            device (str): unused.

        Returns:
            CompletedProcess: the result of predicting subprocess execution.
        """
        # `device` unused
        cli_args = self._gather_opts_args(cli_args)
        cmd = [self.python, "tools/predict.py", "--config", config_path, *cli_args]
        return self.run_cmd(cmd, switch_wdir=True, echo=True, silent=False)

    def export(self, config_path, cli_args, device):
        """export"""
        cmd = [
            self.python,
            "tools/export.py",
            "--config",
            config_path,
            *cli_args,
        ]
        cp = self.run_cmd(cmd, switch_wdir=True, echo=True, silent=False)
        return cp

    def infer(self, config_path, cli_args, device):
        """infer"""
        raise_unsupported_api_error("infer", self.__class__)

    def compression(
        self, config_path, train_cli_args, export_cli_args, device, train_save_dir
    ):
        """compression"""
        raise_unsupported_api_error("compression", self.__class__)

    def _gather_opts_args(self, args):
        # Since `--opts` in PaddleSeg does not use `action='append'`
        # We collect and arrange all opts args here
        # e.g.: python tools/train.py --config xxx --opts a=1 c=3 --opts b=2
        # => python tools/train.py --config xxx c=3 --opts a=1 b=2
        return gather_opts_args(args, "--opts")


def _extract_eval_metrics(stdout: str) -> dict:
    """extract evaluation metrics from training log

    Args:
        stdout (str): the training log

    Returns:
        dict: the training metric
    """
    import re

    pattern = r"\'mse\':\s+(\d+\.\d+),+[\s|\n]+\'mae\':\s+(\d+\.\d+)"
    keys = ["mse", "mae"]

    metric_dict = dict()
    pattern = re.compile(pattern)

    lines = stdout.splitlines()
    for line in lines:
        match = pattern.search(line)
        if match:
            for k, v in zip(keys, map(float, match.groups())):
                metric_dict[k] = v

    return metric_dict



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import sys
import traceback

from .file_interface import write_json_file


def try_except_decorator(func):
    """try-except"""

    def wrap(self, *args, **kwargs):
        try:
            result = func(self, *args, **kwargs)
            if result:
                save_result(True, self._mode, self._output, result_dict=result)
        except Exception:
            exc_type, exc_value, exc_tb = sys.exc_info()
            save_result(
                False,
                self._mode,
                self._output,
                err_type=str(exc_type),
                err_msg=str(exc_value),
            )
            traceback.print_exception(exc_type, exc_value, exc_tb)
            sys.exit(1)

    return wrap


def save_result(run_pass, mode, output, result_dict=None, err_type=None, err_msg=None):
    """format, build and save result"""
    json_data = {"done_flag": run_pass}
    if not run_pass:
        assert result_dict is None and err_type is not None and err_msg is not None
        json_data.update({"err_type": err_type, "err_msg": err_msg})
    else:
        assert result_dict is not None and err_type is None and err_msg is None
        json_data.update(result_dict)
    if not os.path.exists(output):
        os.makedirs(output, exist_ok=True)

    write_json_file(json_data, os.path.join(output, f"{mode}_result.json"), indent=2)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import os.path as osp

from ...base.register import register_model_info, register_suite_info
from .config import DetConfig
from .model import DetModel
from .runner import DetRunner

REPO_ROOT_PATH = os.environ.get("PADDLE_PDX_PADDLEDETECTION_PATH")
PDX_CONFIG_DIR = osp.abspath(osp.join(osp.dirname(__file__), "..", "configs"))

register_suite_info(
    {
        "suite_name": "Det",
        "model": DetModel,
        "runner": DetRunner,
        "config": DetConfig,
        "runner_root_path": REPO_ROOT_PATH,
    }
)

################ Models Using Universal Config ################
register_model_info(
    {
        "model_name": "PicoDet-S",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-S.yaml"),
        "auto_compression_config_path": osp.join(
            PDX_CONFIG_DIR, "slim", "picodet_s_lcnet_qat.yml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "compression"],
        "supported_dataset_types": ["COCODetDataset"],
    }
)

register_model_info(
    {
        "model_name": "PicoDet-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-L.yaml"),
        "auto_compression_config_path": osp.join(
            PDX_CONFIG_DIR, "slim", "picodet_l_lcnet_qat.yml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "compression"],
        "supported_dataset_types": ["COCODetDataset"],
    }
)

register_model_info(
    {
        "model_name": "PP-YOLOE_plus-S",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus-S.yaml"),
        "auto_compression_config_path": osp.join(
            PDX_CONFIG_DIR, "slim", "ppyoloe_plus_crn_s_qat.yml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "compression"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx"],
            "dy2st": False,
            "amp": ["O1", "O2"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-YOLOE_plus-M",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus-M.yaml"),
        "auto_compression_config_path": osp.join(
            PDX_CONFIG_DIR, "slim", "ppyoloe_plus_crn_l_qat.yml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "compression"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx"],
            "dy2st": False,
            "amp": ["O1", "O2"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-YOLOE_plus-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["O1", "O2"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-YOLOE_plus-X",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus-X.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["O1", "O2"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-H",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-H.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-X",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-X.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-R18",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-R18.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-R50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-R50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet_layout_1x",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet_layout_1x.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet_layout_1x_table",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet_layout_1x_table.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOv3-DarkNet53",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOv3-DarkNet53.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOv3-MobileNetV3",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOv3-MobileNetV3.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOv3-ResNet50_vd_DCN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOv3-ResNet50_vd_DCN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOX-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOX-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOX-M",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOX-M.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOX-N",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOX-N.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOX-S",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOX-S.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOX-T",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOX-T.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "YOLOX-X",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "YOLOX-X.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet34-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNet34-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNet50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet50-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNet50-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet50-vd-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNet50-vd-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet50-vd-SSLDv2-FPN",
        "suite": "Det",
        "config_path": osp.join(
            PDX_CONFIG_DIR, "FasterRCNN-ResNet50-vd-SSLDv2-FPN.yaml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet101",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNet101.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNet101-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNet101-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-ResNeXt101-vd-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-ResNeXt101-vd-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FasterRCNN-Swin-Tiny-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FasterRCNN-Swin-Tiny-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "Cascade-FasterRCNN-ResNet50-FPN",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "Cascade-FasterRCNN-ResNet50-FPN.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "Cascade-FasterRCNN-ResNet50-vd-SSLDv2-FPN",
        "suite": "Det",
        "config_path": osp.join(
            PDX_CONFIG_DIR, "Cascade-FasterRCNN-ResNet50-vd-SSLDv2-FPN.yaml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PicoDet-XS",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-XS.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PicoDet-M",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-M.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "FCOS-ResNet50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "FCOS-ResNet50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "DETR-R50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "DETR-R50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE-L_vehicle",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE-L_vehicle.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE-S_vehicle",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE-S_vehicle.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-ShiTuV2_det",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-ShiTuV2_det.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE-L_human",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE-L_human.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE-S_human",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE-S_human.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "CenterNet-DLA-34",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "CenterNet-DLA-34.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "CenterNet-ResNet50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "CenterNet-ResNet50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE_plus_SOD-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus_SOD-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE_plus_SOD-S",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus_SOD-S.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "PP-YOLOE_plus_SOD-largesize-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus_SOD-largesize-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "RT-DETR-H_layout_3cls",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-H_layout_3cls.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet-S_layout_3cls",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-S_layout_3cls.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet-S_layout_17cls",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-S_layout_17cls.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet-L_layout_3cls",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-L_layout_3cls.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet-L_layout_17cls",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet-L_layout_17cls.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)


register_model_info(
    {
        "model_name": "RT-DETR-H_layout_17cls",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-H_layout_17cls.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PicoDet_LCNet_x2_5_face",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PicoDet_LCNet_x2_5_face.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "BlazeFace",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "BlazeFace.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "BlazeFace-FPN-SSH",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "BlazeFace-FPN-SSH.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-YOLOE_plus-S_face",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE_plus-S_face.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-YOLOE-R-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-YOLOE-R-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-L_wired_table_cell_det",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "RT-DETR-L_wired_table_cell_det.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "RT-DETR-L_wireless_table_cell_det",
        "suite": "Det",
        "config_path": osp.join(
            PDX_CONFIG_DIR, "RT-DETR-L_wireless_table_cell_det.yaml"
        ),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "Co-Deformable-DETR-R50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "Co-Deformable-DETR-R50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "Co-Deformable-DETR-Swin-T",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "Co-Deformable-DETR-Swin-T.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "Co-DINO-R50",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "Co-DINO-R50.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "Co-DINO-Swin-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "Co-DINO-Swin-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-TinyPose_128x96",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-TinyPose_128x96.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["KeypointTopDownCocoDetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-TinyPose_256x192",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-TinyPose_256x192.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["KeypointTopDownCocoDetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-DocLayout-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-DocLayout-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-DocLayout-M",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-DocLayout-M.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-DocLayout-S",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-DocLayout-S.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-DocLayout_plus-L",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-DocLayout_plus-L.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)

register_model_info(
    {
        "model_name": "PP-DocBlockLayout",
        "suite": "Det",
        "config_path": osp.join(PDX_CONFIG_DIR, "PP-DocBlockLayout.yaml"),
        "supported_apis": ["train", "evaluate", "predict", "export", "infer"],
        "supported_dataset_types": ["COCODetDataset"],
        "supported_train_opts": {
            "device": ["cpu", "gpu_nxcx", "xpu", "npu", "mlu"],
            "dy2st": False,
            "amp": ["OFF"],
        },
    }
)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path

from ...utils import logging
from ..base import BaseExportor
from .model_list import MODELS


class TSADExportor(BaseExportor):
    """Image Classification Model Exportor"""

    entities = MODELS

    def get_config_path(self, weight_path):
        """
        get config path

        Args:
            weight_path (str): The path to the weight

        Returns:
            config_path (str): The path to the config

        """
        config_path = Path(self.export_config.weight_path).parent.parent / "config.yaml"
        if not config_path.exists():
            logging.warning(
                f"The config file(`{config_path}`) related to weight file(`{weight_path}`) is not exist, use default instead."
            )
            config_path = None
        return config_path



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import codecs

import yaml

from ...utils.misc import abspath
from ..base import BaseConfig


class PP3DConfig(BaseConfig):
    # Refer to https://github.com/PaddlePaddle/Paddle3D/blob/release/1.0/paddle3d/apis/config.py
    def update(self, dict_like_obj):
        def _merge_config_dicts(dict_from, dict_to):
            # According to
            # https://github.com/PaddlePaddle/Paddle3D/blob/3cf884ecbc94330be0e2db780434bb60b9b4fe8c/paddle3d/apis/config.py#L90
            for key, val in dict_from.items():
                if isinstance(val, dict) and key in dict_to:
                    dict_to[key] = _merge_config_dicts(val, dict_to[key])
                else:
                    dict_to[key] = val
            return dict_to

        dict_ = _merge_config_dicts(dict_like_obj, self.dict)
        self.reset_from_dict(dict_)

    def load(self, config_path):
        with codecs.open(config_path, "r", "utf-8") as file:
            dic = yaml.load(file, Loader=yaml.FullLoader)
        dict_ = dic
        self.reset_from_dict(dict_)

    def dump(self, config_path):
        with open(config_path, "w", encoding="utf-8") as f:
            yaml.dump(self.dict, f)

    def update_learning_rate(self, learning_rate):
        if "lr_scheduler" not in self:
            raise RuntimeError(
                "Not able to update learning rate, because no LR scheduler config was found."
            )

        # Some lr_scheduler in Paddle3D has not learning_rate parameter.
        if self.lr_scheduler["type"] == "OneCycle":
            self.lr_scheduler["lr_max"] = learning_rate
        elif self.lr_scheduler["type"] == "OneCycleWarmupDecayLr":
            self.lr_scheduler["base_learning_rate"] = learning_rate
        else:
            self.lr_scheduler["learning_rate"] = learning_rate

    def update_batch_size(self, batch_size, mode="train"):
        if mode == "train":
            self.set_val("batch_size", batch_size)
        else:
            raise ValueError(
                f"Setting `batch_size` in {repr(mode)} mode is not supported."
            )

    def update_epochs(self, epochs, mode="train"):
        if mode == "train":
            self.set_val("epochs", epochs)
        else:
            raise ValueError(f"Setting `epochs` in {repr(mode)} mode is not supported.")

    def update_pretrained_weights(self, weight_path, is_backbone=False):
        raise NotImplementedError

    def get_epochs_iters(self):
        if "iters" in self:
            return self.iters
        else:
            assert "epochs" in self
            return self.epochs

    def get_learning_rate(self):
        if "lr_scheduler" not in self or "learning_rate" not in self.lr_scheduler:
            # Default lr
            return 0.0001
        else:
            lr = self.lr_scheduler["learning_rate"]
            while isinstance(lr, dict):
                lr = lr["learning_rate"]
            return lr

    def get_batch_size(self, mode="train"):
        if "batch_size" in self:
            return self.batch_size
        else:
            # Default batch size
            return 1

    def get_qat_epochs_iters(self):
        assert (
            "finetune_config" in self
        ), "QAT training yaml should contain finetune_config key"
        if "iters" in self.finetune_config:
            return self.finetune_config["iters"]
        else:
            assert "epochs" in self.finetune_config
            return self.finetune_config["epochs"]

    def get_qat_learning_rate(self):
        assert (
            "finetune_config" in self
        ), "QAT training yaml should contain finetune_config key"
        cfg = self.finetune_config
        if "lr_scheduler" in cfg or "learning_rate" not in cfg.lr_scheduler:
            # Default lr
            return 1.25e-4
        else:
            lr = cfg.lr_scheduler["learning_rate"]
            while isinstance(lr, dict):
                lr = lr["learning_rate"]
            return lr

    def update_warmup_steps(self, steps):
        self.lr_scheduler["warmup_steps"] = steps

    def update_end_lr(self, learning_rate):
        self.lr_scheduler["end_lr"] = learning_rate

    def update_iters(self, iters):
        self.set_val("iters", iters)
        if "epochs" in self:
            self.set_val("epochs", None)

    def update_finetune_iters(self, iters):
        self.finetune_config["iters"] = iters
        if "epochs" in self.finetune_config:
            self.finetune_config["epochs"] = None

    def update_save_dir(self, save_dir: str):
        self["save_dir"] = abspath(save_dir)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import numpy as np
from PIL import ImageDraw

from ......utils.deps import function_requires_deps, is_dep_available

if is_dep_available("pycocotools"):
    from pycocotools.coco import COCO


@function_requires_deps("pycocotools")
def draw_keypoint(image, coco_info: "COCO", img_id):
    """
    Draw keypoints on image for the COCO human keypoint dataset with 17 keypoints.

    Args:
        image: PIL.Image object
        coco_info: COCO object (from pycocotools.coco)
        img_id: Image ID

    Returns:
        image: PIL.Image object with keypoints and skeleton drawn
    """

    # Initialize the drawing context
    image = image.convert("RGB")
    draw = ImageDraw.Draw(image)
    image_size = image.size
    width = int(max(image_size) * 0.005)  # Line thickness for drawing

    # Define the skeleton connections based on COCO keypoint indexes
    skeleton = [
        [15, 13],
        [13, 11],
        [16, 14],
        [14, 12],
        [11, 12],
        [5, 11],
        [6, 12],
        [5, 6],
        [5, 7],
        [6, 8],
        [7, 9],
        [8, 10],
        [1, 2],
        [0, 1],
        [0, 2],
        [1, 3],
        [2, 4],
        [3, 5],
        [4, 6],
    ]

    # Define colors for each keypoint (you can customize these colors)
    keypoint_colors = [
        (255, 0, 0),  # Nose
        (255, 85, 0),  # Left Eye
        (255, 170, 0),  # Right Eye
        (255, 255, 0),  # Left Ear
        (170, 255, 0),  # Right Ear
        (85, 255, 0),  # Left Shoulder
        (0, 255, 0),  # Right Shoulder
        (0, 255, 85),  # Left Elbow
        (0, 255, 170),  # Right Elbow
        (0, 255, 255),  # Left Wrist
        (0, 170, 255),  # Right Wrist
        (0, 85, 255),  # Left Hip
        (0, 0, 255),  # Right Hip
        (85, 0, 255),  # Left Knee
        (170, 0, 255),  # Right Knee
        (255, 0, 255),  # Left Ankle
        (255, 0, 170),  # Right Ankle
    ]

    # Get annotations for the image
    annotations = coco_info.loadAnns(coco_info.getAnnIds(imgIds=img_id))

    # Loop over each person annotation
    for ann in annotations:
        keypoints = ann.get("keypoints", [])
        if not keypoints:
            continue  # Skip if no keypoints are present

        # Reshape keypoints into (num_keypoints, 3)
        keypoints = np.array(keypoints).reshape(-1, 3)

        # Draw keypoints
        for idx, (x, y, v) in enumerate(keypoints):
            if v == 2:  # v=2 means the keypoint is labeled and visible
                radius = max(1, int(width / 2))
                x, y = float(x), float(y)
                color = keypoint_colors[idx % len(keypoint_colors)]
                draw.ellipse(
                    (x - radius, y - radius, x + radius, y + radius), fill=color
                )

        # Draw skeleton by connecting keypoints
        for sk in skeleton:
            kp1_idx, kp2_idx = sk[0], sk[1]
            x1, y1, v1 = keypoints[kp1_idx]
            x2, y2, v2 = keypoints[kp2_idx]
            if v1 == 2 and v2 == 2:
                # Both keypoints are visible
                x1, y1 = float(x1), float(y1)
                x2, y2 = float(x2), float(y2)
                draw.line(
                    (x1, y1, x2, y2),
                    fill=(0, 255, 0),  # Line color (you can customize)
                    width=width,
                )

    return image



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import logging
import os

import chardet
import ruamel.yaml
import yaml
from filelock import FileLock

try:
    import ujson as json
except:
    logging.warning("failed to import ujson, using json instead")
    import json

from contextlib import contextmanager


@contextmanager
def custom_open(file_path, mode):
    """
    自定义打开文件函数

    Args:
        file_path (str): 文件路径
        mode (str): 文件打开模式，'r'，'w' 或 'a'

    Returns:
        Any: 返回文件对象

    Raises:
        FileNotFoundError: 当文件不存在时，raise FileNotFoundError
        ValueError: 当 mode 参数不是 'r'， 'w' 和 'a' 时，raise ValueError
    """
    if mode == "r":
        if not os.path.exists(file_path):
            raise FileNotFoundError("file {} not found".format(file_path))
        file = open(file_path, "r", encoding="utf-8")
        try:
            file.read()
            file.seek(0)
            yield file
        except UnicodeDecodeError:
            file = open(file_path, "r", encoding="gbk")
            try:
                file.read()
                file.seek(0)
                yield file
            except UnicodeDecodeError:
                with open(file_path, "rb") as f:
                    encoding = chardet.detect(f.read())["encoding"]
                file = open(file_path, "r", encoding=encoding)
                yield file
        finally:
            file.close()
    elif mode == "w":
        file = open(file_path, "w", encoding="utf-8")
        yield file
        file.close()
    elif mode == "a":
        encoding = "utf-8"
        if os.path.exists(file_path):
            file = open(file_path, "r", encoding=encoding)
            try:
                file.read()
                file.seek(0)
            except UnicodeDecodeError:
                encoding = "gbk"
                file = open(file_path, "r", encoding=encoding)
                try:
                    file.read()
                    file.seek(0)
                except UnicodeDecodeError:
                    with open(file_path, "rb") as f:
                        encoding = chardet.detect(f.read())["encoding"]
            finally:
                file.close()

        file = open(file_path, "a", encoding=encoding)
        yield file
        file.close()
    else:
        raise ValueError("mode must be 'r', 'w' or 'a', but got {}".format(mode))


# --------------- yaml ---------------


def read_yaml_file(yaml_path: str, to_dict=True):
    """read from yaml file"""
    try:
        with open(yaml_path, "r", encoding="utf-8") as file:
            yaml_content = yaml.full_load(file)
    except UnicodeDecodeError:
        with open(yaml_path, "r", encoding="gbk") as file:
            yaml_content = yaml.full_load(file)
    yaml_content = dict(yaml_content) if to_dict else yaml_content
    return yaml_content


def write_config_file(yaml_dict: dict, yaml_path: str):
    """write to config yaml file"""
    yaml = ruamel.yaml.YAML()
    lock = FileLock(yaml_path + ".lock")
    with lock:
        with open(yaml_path, "w", encoding="utf-8") as file:
            # yaml.safe_dump(yaml_dict, file, sort_keys=False)
            yaml.dump(yaml_dict, file)


def update_yaml_file_with_dict(yaml_path, key_values: dict):
    """update yaml file with key_values
    key_values is a dict
    """
    yaml_dict = read_yaml_file(yaml_path)
    yaml_dict.update(key_values)
    write_config_file(yaml_dict, yaml_path)


def get_yaml_keys(yaml_path):
    """get all keys of yaml file"""
    yaml_dict = read_yaml_file(yaml_path)
    return yaml_dict.keys()


# --------------- markdown ---------------


def generate_markdown_from_dict(metrics):
    """generate_markdown_from_dict"""
    mk = ""
    keys = metrics.keys()
    mk += "| ".join(keys())
    mk += os.linesep
    mk += "|".join([" :----: "])


# ------------------- jsonl ---------------------


def read_jsonl_file(jsonl_path: str):
    """read from jsonl file"""
    with custom_open(jsonl_path, "r") as file:
        jsonl_content = [json.loads(line) for line in file]
    return jsonl_content


def write_json_file(content, jsonl_path: str, ensure_ascii=False, **kwargs):
    """write to json file"""
    with custom_open(jsonl_path, "w") as file:
        json.dump(content, file, ensure_ascii=ensure_ascii, **kwargs)


# --------------- check webui yaml -----------------


def check_dict_keys(to_checked_dict, standard_dict, escape_list=None):
    """check if all keys of to_checked_dict is the same as standard_dict, and the value is the same type
    Args:
        escape_list: if set, will not check the keys in white_list
    """
    escape_list = [] if escape_list is None else escape_list
    for key in standard_dict.keys():
        if key not in to_checked_dict:
            logging.error(f"key {key} not in yaml file")
            return False
        if not isinstance(standard_dict[key], type(to_checked_dict[key])):
            logging.error(
                f"value type of key {key} is not the same as standard: "
                f"{type(standard_dict[key])}, {type(to_checked_dict[key])}"
            )
            return False

        if (
            isinstance(standard_dict[key], dict)
            and isinstance(to_checked_dict[key], dict)
            and key not in escape_list
        ):
            return check_dict_keys(
                to_checked_dict[key], standard_dict[key], escape_list
            )
    if len(to_checked_dict.keys()) != len(standard_dict.keys()):
        logging.error(f"yaml file has extra keys")
        return False

    return True


def check_dataset_valid(path_list):
    """check if dataset valid in path_list for datset_ui"""
    if path_list is not None and len(path_list) > 0:
        for path in path_list:
            if not os.path.exists(path):
                return False
        return True
    else:
        return False



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Generic, List, Tuple, TypeVar, Union

from pydantic import BaseModel, Discriminator
from typing_extensions import Annotated, Literal, TypeAlias

from ....utils.deps import is_dep_available

if is_dep_available("openai"):
    from openai.types.chat import ChatCompletion

__all__ = [
    "AIStudioNoResultResponse",
    "ResultT",
    "AIStudioResultResponse",
    "Response",
    "ImageInfo",
    "PDFPageInfo",
    "PDFInfo",
    "DataInfo",
    "PrimaryOperations",
]


class AIStudioNoResultResponse(BaseModel):
    logId: str
    errorCode: int
    errorMsg: str


ResultT = TypeVar("ResultT", bound=BaseModel)


class AIStudioResultResponse(BaseModel, Generic[ResultT]):
    logId: str
    result: ResultT
    errorCode: Literal[0] = 0
    errorMsg: Literal["Success"] = "Success"


Response: TypeAlias = Union[
    AIStudioResultResponse, AIStudioNoResultResponse, "ChatCompletion"
]


class ImageInfo(BaseModel):
    width: int
    height: int
    type: Literal["image"] = "image"


class PDFPageInfo(BaseModel):
    width: int
    height: int


class PDFInfo(BaseModel):
    numPages: int
    pages: List[PDFPageInfo]
    type: Literal["pdf"] = "pdf"


DataInfo: TypeAlias = Annotated[Union[ImageInfo, PDFInfo], Discriminator("type")]

# Should we use generics?
PrimaryOperations: TypeAlias = Dict[str, Tuple[str, BaseModel, BaseModel]]



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil
import tempfile
from abc import ABC, abstractmethod
from pathlib import Path

import huggingface_hub as hf_hub

hf_hub.logging.set_verbosity_error()

import modelscope
import requests

os.environ["AISTUDIO_LOG"] = "critical"
from aistudio_sdk.snapshot_download import snapshot_download as aistudio_download

from ...utils import logging
from ...utils.cache import CACHE_DIR
from ...utils.download import download_and_extract
from ...utils.flags import MODEL_SOURCE

ALL_MODELS = [
    "ResNet18",
    "ResNet18_vd",
    "ResNet34",
    "ResNet34_vd",
    "ResNet50",
    "ResNet50_vd",
    "ResNet101",
    "ResNet101_vd",
    "ResNet152",
    "ResNet152_vd",
    "ResNet200_vd",
    "PP-LCNet_x0_25",
    "PP-LCNet_x0_25_textline_ori",
    "PP-LCNet_x0_35",
    "PP-LCNet_x0_5",
    "PP-LCNet_x0_75",
    "PP-LCNet_x1_0",
    "PP-LCNet_x1_0_doc_ori",
    "PP-LCNet_x1_0_textline_ori",
    "PP-LCNet_x1_5",
    "PP-LCNet_x2_5",
    "PP-LCNet_x2_0",
    "PP-LCNetV2_small",
    "PP-LCNetV2_base",
    "PP-LCNetV2_large",
    "MobileNetV3_large_x0_35",
    "MobileNetV3_large_x0_5",
    "MobileNetV3_large_x0_75",
    "MobileNetV3_large_x1_0",
    "MobileNetV3_large_x1_25",
    "MobileNetV3_small_x0_35",
    "MobileNetV3_small_x0_5",
    "MobileNetV3_small_x0_75",
    "MobileNetV3_small_x1_0",
    "MobileNetV3_small_x1_25",
    "ConvNeXt_tiny",
    "ConvNeXt_small",
    "ConvNeXt_base_224",
    "ConvNeXt_base_384",
    "ConvNeXt_large_224",
    "ConvNeXt_large_384",
    "MobileNetV2_x0_25",
    "MobileNetV2_x0_5",
    "MobileNetV2_x1_0",
    "MobileNetV2_x1_5",
    "MobileNetV2_x2_0",
    "MobileNetV1_x0_25",
    "MobileNetV1_x0_5",
    "MobileNetV1_x0_75",
    "MobileNetV1_x1_0",
    "SwinTransformer_tiny_patch4_window7_224",
    "SwinTransformer_small_patch4_window7_224",
    "SwinTransformer_base_patch4_window7_224",
    "SwinTransformer_base_patch4_window12_384",
    "SwinTransformer_large_patch4_window7_224",
    "SwinTransformer_large_patch4_window12_384",
    "PP-HGNet_tiny",
    "PP-HGNet_small",
    "PP-HGNet_base",
    "PP-HGNetV2-B0",
    "PP-HGNetV2-B1",
    "PP-HGNetV2-B2",
    "PP-HGNetV2-B3",
    "PP-HGNetV2-B4",
    "PP-HGNetV2-B5",
    "PP-HGNetV2-B6",
    "FasterNet-L",
    "FasterNet-M",
    "FasterNet-S",
    "FasterNet-T0",
    "FasterNet-T1",
    "FasterNet-T2",
    "StarNet-S1",
    "StarNet-S2",
    "StarNet-S3",
    "StarNet-S4",
    "MobileNetV4_conv_small",
    "MobileNetV4_conv_medium",
    "MobileNetV4_conv_large",
    "MobileNetV4_hybrid_medium",
    "MobileNetV4_hybrid_large",
    "CLIP_vit_base_patch16_224",
    "CLIP_vit_large_patch14_224",
    "PP-LCNet_x1_0_ML",
    "PP-HGNetV2-B0_ML",
    "PP-HGNetV2-B4_ML",
    "PP-HGNetV2-B6_ML",
    "ResNet50_ML",
    "CLIP_vit_base_patch16_448_ML",
    "PP-YOLOE_plus-X",
    "PP-YOLOE_plus-L",
    "PP-YOLOE_plus-M",
    "PP-YOLOE_plus-S",
    "RT-DETR-L",
    "RT-DETR-H",
    "RT-DETR-X",
    "YOLOv3-DarkNet53",
    "YOLOv3-MobileNetV3",
    "YOLOv3-ResNet50_vd_DCN",
    "YOLOX-L",
    "YOLOX-M",
    "YOLOX-N",
    "YOLOX-S",
    "YOLOX-T",
    "YOLOX-X",
    "RT-DETR-R18",
    "RT-DETR-R50",
    "PicoDet-S",
    "PicoDet-L",
    "Deeplabv3-R50",
    "Deeplabv3-R101",
    "Deeplabv3_Plus-R50",
    "Deeplabv3_Plus-R101",
    "PP-ShiTuV2_rec",
    "PP-ShiTuV2_rec_CLIP_vit_base",
    "PP-ShiTuV2_rec_CLIP_vit_large",
    "PP-LiteSeg-T",
    "PP-LiteSeg-B",
    "OCRNet_HRNet-W48",
    "OCRNet_HRNet-W18",
    "SegFormer-B0",
    "SegFormer-B1",
    "SegFormer-B2",
    "SegFormer-B3",
    "SegFormer-B4",
    "SegFormer-B5",
    "SeaFormer_tiny",
    "SeaFormer_small",
    "SeaFormer_base",
    "SeaFormer_large",
    "Mask-RT-DETR-H",
    "Mask-RT-DETR-L",
    "PP-OCRv4_server_rec",
    "Mask-RT-DETR-S",
    "Mask-RT-DETR-M",
    "Mask-RT-DETR-X",
    "SOLOv2",
    "MaskRCNN-ResNet50",
    "MaskRCNN-ResNet50-FPN",
    "MaskRCNN-ResNet50-vd-FPN",
    "MaskRCNN-ResNet101-FPN",
    "MaskRCNN-ResNet101-vd-FPN",
    "MaskRCNN-ResNeXt101-vd-FPN",
    "Cascade-MaskRCNN-ResNet50-FPN",
    "Cascade-MaskRCNN-ResNet50-vd-SSLDv2-FPN",
    "PP-YOLOE_seg-S",
    "PP-OCRv3_mobile_rec",
    "en_PP-OCRv3_mobile_rec",
    "korean_PP-OCRv3_mobile_rec",
    "japan_PP-OCRv3_mobile_rec",
    "chinese_cht_PP-OCRv3_mobile_rec",
    "te_PP-OCRv3_mobile_rec",
    "ka_PP-OCRv3_mobile_rec",
    "ta_PP-OCRv3_mobile_rec",
    "latin_PP-OCRv3_mobile_rec",
    "arabic_PP-OCRv3_mobile_rec",
    "cyrillic_PP-OCRv3_mobile_rec",
    "devanagari_PP-OCRv3_mobile_rec",
    "en_PP-OCRv4_mobile_rec",
    "PP-OCRv4_server_rec_doc",
    "PP-OCRv4_mobile_rec",
    "PP-OCRv4_server_det",
    "PP-OCRv4_mobile_det",
    "PP-OCRv3_server_det",
    "PP-OCRv3_mobile_det",
    "PP-OCRv4_server_seal_det",
    "PP-OCRv4_mobile_seal_det",
    "ch_RepSVTR_rec",
    "ch_SVTRv2_rec",
    "PP-LCNet_x1_0_pedestrian_attribute",
    "PP-LCNet_x1_0_vehicle_attribute",
    "PicoDet_layout_1x",
    "PicoDet_layout_1x_table",
    "SLANet",
    "SLANet_plus",
    "LaTeX_OCR_rec",
    "UniMERNet",
    "PP-FormulaNet-S",
    "PP-FormulaNet-L",
    "PP-FormulaNet_plus-S",
    "PP-FormulaNet_plus-M",
    "PP-FormulaNet_plus-L",
    "FasterRCNN-ResNet34-FPN",
    "FasterRCNN-ResNet50",
    "FasterRCNN-ResNet50-FPN",
    "FasterRCNN-ResNet50-vd-FPN",
    "FasterRCNN-ResNet50-vd-SSLDv2-FPN",
    "FasterRCNN-ResNet101",
    "FasterRCNN-ResNet101-FPN",
    "FasterRCNN-ResNeXt101-vd-FPN",
    "FasterRCNN-Swin-Tiny-FPN",
    "Cascade-FasterRCNN-ResNet50-FPN",
    "Cascade-FasterRCNN-ResNet50-vd-SSLDv2-FPN",
    "UVDoc",
    "DLinear",
    "NLinear",
    "RLinear",
    "Nonstationary",
    "TimesNet",
    "TiDE",
    "PatchTST",
    "DLinear_ad",
    "AutoEncoder_ad",
    "Nonstationary_ad",
    "PatchTST_ad",
    "TimesNet_ad",
    "TimesNet_cls",
    "STFPM",
    "FCOS-ResNet50",
    "DETR-R50",
    "PP-YOLOE-L_vehicle",
    "PP-YOLOE-S_vehicle",
    "PP-ShiTuV2_det",
    "PP-YOLOE-S_human",
    "PP-YOLOE-L_human",
    "PicoDet-M",
    "PicoDet-XS",
    "PP-YOLOE_plus_SOD-L",
    "PP-YOLOE_plus_SOD-S",
    "PP-YOLOE_plus_SOD-largesize-L",
    "CenterNet-DLA-34",
    "CenterNet-ResNet50",
    "PicoDet-S_layout_3cls",
    "PicoDet-S_layout_17cls",
    "PicoDet-L_layout_3cls",
    "PicoDet-L_layout_17cls",
    "RT-DETR-H_layout_3cls",
    "RT-DETR-H_layout_17cls",
    "PicoDet_LCNet_x2_5_face",
    "BlazeFace",
    "BlazeFace-FPN-SSH",
    "PP-YOLOE_plus-S_face",
    "MobileFaceNet",
    "ResNet50_face",
    "PP-YOLOE-R-L",
    "Co-Deformable-DETR-R50",
    "Co-Deformable-DETR-Swin-T",
    "Co-DINO-R50",
    "Co-DINO-Swin-L",
    "whisper_large",
    "whisper_base",
    "whisper_medium",
    "whisper_small",
    "whisper_tiny",
    "PP-TSM-R50_8frames_uniform",
    "PP-TSMv2-LCNetV2_8frames_uniform",
    "PP-TSMv2-LCNetV2_16frames_uniform",
    "MaskFormer_tiny",
    "MaskFormer_small",
    "PP-LCNet_x1_0_table_cls",
    "SLANeXt_wired",
    "SLANeXt_wireless",
    "RT-DETR-L_wired_table_cell_det",
    "RT-DETR-L_wireless_table_cell_det",
    "YOWO",
    "PP-TinyPose_128x96",
    "PP-TinyPose_256x192",
    "GroundingDINO-T",
    "SAM-H_box",
    "SAM-H_point",
    "PP-DocLayout-L",
    "PP-DocLayout-M",
    "PP-DocLayout-S",
    "PP-DocLayout_plus-L",
    "PP-DocBlockLayout",
    "BEVFusion",
    "YOLO-Worldv2-L",
    "PP-DocBee-2B",
    "PP-DocBee-7B",
    "PP-Chart2Table",
    "PP-OCRv5_server_det",
    "PP-OCRv5_mobile_det",
    "PP-OCRv5_server_rec",
    "PP-OCRv5_mobile_rec",
    "eslav_PP-OCRv5_mobile_rec",
    "PP-DocBee2-3B",
    "latin_PP-OCRv5_mobile_rec",
    "korean_PP-OCRv5_mobile_rec",
    "th_PP-OCRv5_mobile_rec",
    "el_PP-OCRv5_mobile_rec",
    "en_PP-OCRv5_mobile_rec",
]


OCR_MODELS = [
    "arabic_PP-OCRv3_mobile_rec",
    "chinese_cht_PP-OCRv3_mobile_rec",
    "ch_RepSVTR_rec",
    "ch_SVTRv2_rec",
    "cyrillic_PP-OCRv3_mobile_rec",
    "devanagari_PP-OCRv3_mobile_rec",
    "en_PP-OCRv3_mobile_rec",
    "en_PP-OCRv4_mobile_rec",
    "eslav_PP-OCRv5_mobile_rec",
    "japan_PP-OCRv3_mobile_rec",
    "ka_PP-OCRv3_mobile_rec",
    "korean_PP-OCRv3_mobile_rec",
    "korean_PP-OCRv5_mobile_rec",
    "LaTeX_OCR_rec",
    "latin_PP-OCRv3_mobile_rec",
    "latin_PP-OCRv5_mobile_rec",
    "en_PP-OCRv5_mobile_rec",
    "th_PP-OCRv5_mobile_rec",
    "el_PP-OCRv5_mobile_rec",
    "PicoDet_layout_1x",
    "PicoDet_layout_1x_table",
    "PicoDet-L_layout_17cls",
    "PicoDet-L_layout_3cls",
    "PicoDet-S_layout_17cls",
    "PicoDet-S_layout_3cls",
    "PP-DocBee2-3B",
    "PP-Chart2Table",
    "PP-DocBee-2B",
    "PP-DocBee-7B",
    "PP-DocBlockLayout",
    "PP-DocLayout-L",
    "PP-DocLayout-M",
    "PP-DocLayout_plus-L",
    "PP-DocLayout-S",
    "PP-FormulaNet-L",
    "PP-FormulaNet_plus-L",
    "PP-FormulaNet_plus-M",
    "PP-FormulaNet_plus-S",
    "PP-FormulaNet-S",
    "PP-LCNet_x0_25_textline_ori",
    "PP-LCNet_x1_0_doc_ori",
    "PP-LCNet_x1_0_table_cls",
    "PP-LCNet_x1_0_textline_ori",
    "PP-OCRv3_mobile_det",
    "PP-OCRv3_mobile_rec",
    "PP-OCRv3_server_det",
    "PP-OCRv4_mobile_det",
    "PP-OCRv4_mobile_rec",
    "PP-OCRv4_mobile_seal_det",
    "PP-OCRv4_server_det",
    "PP-OCRv4_server_rec_doc",
    "PP-OCRv4_server_rec",
    "PP-OCRv4_server_seal_det",
    "PP-OCRv5_mobile_det",
    "PP-OCRv5_mobile_rec",
    "PP-OCRv5_server_det",
    "PP-OCRv5_server_rec",
    "RT-DETR-H_layout_17cls",
    "RT-DETR-H_layout_3cls",
    "RT-DETR-L_wired_table_cell_det",
    "RT-DETR-L_wireless_table_cell_det",
    "SLANet",
    "SLANet_plus",
    "SLANeXt_wired",
    "SLANeXt_wireless",
    "ta_PP-OCRv3_mobile_rec",
    "te_PP-OCRv3_mobile_rec",
    "UniMERNet",
    "UVDoc",
]


class _BaseModelHoster(ABC):
    alias = ""
    model_list = []
    healthcheck_url = None
    _healthcheck_timeout = 1

    def __init__(self, save_dir):
        self._save_dir = save_dir

    def get_model(self, model_name):
        assert (
            model_name in self.model_list
        ), f"The model {model_name} is not supported on hosting {self.__class__.__name__}!"
        model_dir = self._save_dir / f"{model_name}"
        if os.path.exists(model_dir):
            logging.info(
                f"Model files already exist. Using cached files. To redownload, please delete the directory manually: `{model_dir}`."
            )
        else:
            logging.info(
                f"Using official model ({model_name}), the model files will be automatically downloaded and saved in `{model_dir}`."
            )
            self._download(model_name, model_dir)
        return model_dir

    @abstractmethod
    def _download(self):
        raise NotImplementedError

    @classmethod
    def is_available(cls):
        if cls.healthcheck_url is None:
            return True
        try:
            response = requests.head(
                cls.healthcheck_url, timeout=cls._healthcheck_timeout
            )
            return response.ok == True
        except Exception:
            logging.debug(f"The model hosting platform({cls.__name__}) is unreachable!")
            return False


class _BosModelHoster(_BaseModelHoster):
    model_list = ALL_MODELS
    alias = "bos"
    healthcheck_url = "https://paddle-model-ecology.bj.bcebos.com"

    version = "paddle3.0.0"
    base_url = (
        "https://paddle-model-ecology.bj.bcebos.com/paddlex/official_inference_model"
    )
    special_model_fn = {
        "whisper_large": "whisper_large.tar",
        "whisper_base": "whisper_base.tar",
        "whisper_medium": "whisper_medium.tar",
        "whisper_small": "whisper_small.tar",
        "whisper_tiny": "whisper_tiny.tar",
    }

    def _download(self, model_name, save_dir):
        if model_name in self.special_model_fn:
            fn = self.special_model_fn[model_name]
        else:
            fn = f"{model_name}_infer.tar"
        url = f"{self.base_url}/{self.version}/{fn}"
        download_and_extract(url, save_dir.parent, model_name, overwrite=False)


class _HuggingFaceModelHoster(_BaseModelHoster):
    model_list = OCR_MODELS
    alias = "huggingface"
    healthcheck_url = "https://huggingface.co"

    def _download(self, model_name, save_dir):
        def _clone(local_dir):
            hf_hub.snapshot_download(
                repo_id=f"PaddlePaddle/{model_name}", local_dir=local_dir
            )

        if os.path.exists(save_dir):
            _clone(save_dir)
        else:
            with tempfile.TemporaryDirectory() as td:
                temp_dir = os.path.join(td, "temp_dir")
                _clone(temp_dir)
                shutil.move(temp_dir, save_dir)


class _ModelScopeModelHoster(_BaseModelHoster):
    model_list = OCR_MODELS
    alias = "modelscope"
    healthcheck_url = "https://modelscope.cn"

    def _download(self, model_name, save_dir):
        def _clone(local_dir):
            modelscope.snapshot_download(
                repo_id=f"PaddlePaddle/{model_name}", local_dir=local_dir
            )

        if os.path.exists(save_dir):
            _clone(save_dir)
        else:
            with tempfile.TemporaryDirectory() as td:
                temp_dir = os.path.join(td, "temp_dir")
                _clone(temp_dir)
                shutil.move(temp_dir, save_dir)


class _AIStudioModelHoster(_BaseModelHoster):
    model_list = OCR_MODELS
    alias = "aistudio"
    healthcheck_url = "https://aistudio.baidu.com"

    def _download(self, model_name, save_dir):
        def _clone(local_dir):
            aistudio_download(repo_id=f"PaddleX/{model_name}", local_dir=local_dir)

        if os.path.exists(save_dir):
            _clone(save_dir)
        else:
            with tempfile.TemporaryDirectory() as td:
                temp_dir = os.path.join(td, "temp_dir")
                _clone(temp_dir)
                shutil.move(temp_dir, save_dir)


class _ModelManager:
    model_list = ALL_MODELS
    _save_dir = Path(CACHE_DIR) / "official_models"

    def __init__(self) -> None:
        self._hosters = self._build_hosters()

    def _build_hosters(self):
        hosters = []
        for hoster_cls in [
            _HuggingFaceModelHoster,
            _AIStudioModelHoster,
            _ModelScopeModelHoster,
            _BosModelHoster,
        ]:
            if hoster_cls.alias == MODEL_SOURCE:
                if hoster_cls.is_available():
                    hosters.insert(0, hoster_cls(self._save_dir))
            else:
                if hoster_cls.is_available():
                    hosters.append(hoster_cls(self._save_dir))
        if len(hosters) == 0:
            logging.warning(
                f"""No model hoster is available! Please check your network connection to one of the following model hosts:
HuggingFace ({_HuggingFaceModelHoster.healthcheck_url}),
ModelScope ({_ModelScopeModelHoster.healthcheck_url}),
AIStudio ({_AIStudioModelHoster.healthcheck_url}), or
BOS ({_BosModelHoster.healthcheck_url}).
Otherwise, only local models can be used."""
            )
        return hosters

    def _get_model_local_path(self, model_name):
        if len(self._hosters) == 0:
            msg = "No available model hosting platforms detected. Please check your network connection."
            logging.error(msg)
            raise Exception(msg)
        return self._download_from_hoster(self._hosters, model_name)

    def _download_from_hoster(self, hosters, model_name):
        for idx, hoster in enumerate(hosters):
            if model_name in hoster.model_list:
                try:
                    return hoster.get_model(model_name)
                except Exception as e:
                    logging.warning(
                        f"Encounter exception when download model from {hoster.alias}: \n{e}."
                    )
                    if len(hosters) <= 1:
                        raise Exception(
                            f"No model source is available! Please check network or use local model files!"
                        )
                    logging.warning(
                        f"PaddleX would try to download from other model sources."
                    )
                    return self._download_from_hoster(hosters[idx + 1 :], model_name)

    def __contains__(self, model_name):
        return model_name in self.model_list

    def __getitem__(self, model_name):
        return self._get_model_local_path(model_name)


official_models = _ModelManager()
